import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

from .regions import regions
from .spots import Spots, get_animation
from .constants import FLUX_SCALE

def sun_test():
    ani = star_test()
    return ani

def star_test(
    activity_rate=1, 
    cycle_length=11, 
    cycle_overlap=2, 
    inclination=1,
    spot_min=7, 
    spot_max=35, 
    period=24.5, 
    shear=0.2, 
    decay_time=5, 
    butterfly=True, 
    t1=0, 
    t2=3650, 
    tstep=1
):
    print('Generating spot evolution for the star...')
    star = regions(
        butterfly=butterfly, 
        activity_rate=activity_rate, 
        cycle_length=cycle_length, 
        cycle_overlap=cycle_overlap, 
        decay_time=(period*decay_time),
        max_ave_lat=spot_max, 
        min_ave_lat=spot_min,
        alpha_med=(activity_rate*FLUX_SCALE)
    )
    spots = Spots(
        star, 
        alpha_med=(activity_rate*FLUX_SCALE), 
        period=period,
        incl=inclination, 
        decay_timescale=decay_time, 
        diffrot_shear=shear
    )
    
    print('Generating light curve...')
    time = np.arange(t1, t2, tstep)
    flux = 1 + spots.calc(time)

    lc = pd.DataFrame(np.vstack([time, flux]).T, columns=['time', 'flux'])
    ani = spots.ortho_animation(time, lc)
    plt.show()

    return ani

def test_animation(path, t1=1000, t2=1365, tstep=1, verbose=False):
    time = np.arange(t1, t2, tstep)
    ani = get_animation(
        path,
        time,
        projection='ortho',
        window_size=100,
        interval=20,
        verbose=verbose,
    )
    plt.show()

    #ani.save("/home/zach/Desktop/lightcurve.gif", writer="imagemagick", dpi=100, fps=10) 
    return ani


from time import time
import numpy as np
from pandas import DataFrame, Series
from scipy.stats import truncnorm
from .constants import RAD2DEG, YEAR2DAY, FLUX_SCALE
import matplotlib.pyplot as plt

import logging

spot_contrast = 0.75

n_bins = 5  # number of area bins
delta_lnA = 0.5  # bin width in log-area
max_area = 100  # original area of largest bipoles (deg^2)

tau1 = 5  # first and last times (in days) for emergence of "correlated" regions
tau2 = 15  # why these numbers ??
prob = 0.001  # total probability for "correlation" # based on what ??
nlon = 36  # number of longitude bins
nlat = 16  # number of latitude bins

dcon = 2 * np.sinh(delta_lnA / 2)  # constant from integration over area bin

fact = np.exp(
    delta_lnA * np.arange(n_bins)
)  # array of area reduction factors, = [1, 1.64, 2.71, 4.48, 7.39]
ftot = fact.sum()  # sum of reduction factors
areas = max_area / fact
bipole_widths = np.sqrt(areas)  # array of bipole widths (deg)


def active_latitudes(min_ave_lat, max_ave_lat, phase, butterfly=True):
    if butterfly:
        return exponential_latitudes(min_ave_lat, max_ave_lat, phase)
    return random_latitudes(min_ave_lat, max_ave_lat, phase)


def exponential_latitudes(min_lat, max_lat, phase):
    # Based on Hathaway 2015, LRSP 12: 4
    phase_scale = 1 / np.log(max_lat / min_lat)
    lat_avg = max_lat * np.exp(-phase / phase_scale)
    # See Hathaway 2010, p. 37 for a discussion of the width
    lat_rms = max_lat / 5 - phase * (max_lat - min_lat) / 7

    return lat_avg, lat_rms


def random_latitudes(min_lat, max_lat, phase):
    lat_avg = (max_lat + min_lat) / 2 * np.ones_like(phase)
    lat_rms = (max_lat - min_lat) * np.ones_like(phase)

    return lat_avg, lat_rms


def regions(
    butterfly=True,
    activity_rate=1,
    cycle_length=11,
    cycle_overlap=2,
    max_ave_lat=35,
    min_ave_lat=7,
    decay_time=120,
    alpha_med=FLUX_SCALE,
    tsim=3650,
    tstart=0,
):
    """ 
    Simulates the emergence and evolution of starspots. 
    Output is a list of active regions.

    PARAMETERS
    ----------
    butterfly = bool - have spots decrease from maxlat to minlat or be randomly located in latitude

    activityrate = Number of magnetic bipoles, normalized such that for the Sun, activityrate = 1.

    cycle_length - length of cycle in years (Sun is 11)

    cycle_overlap - overlap of cycles in years

    max_ave_lat = maximum average latitude of spot emergence (deg)

    min_ave_lat = minimum average latitutde of emergence (deg)

    tsim = how many days to emerge spots for

    tstart = First day to simulate bipoles

    Based on Section 4 of van Ballegooijen 1998, ApJ 501: 866
    and Schrijver and Harvey 1994, SoPh 150: 1S
    Written by Joe Llama (joe.llama@lowell.edu) V 11/1/16
    # Converted to Python 3 9/5/2017

    According to Schrijver and Harvey (1994), the number of active regions
    emerging with areas in the range [A, A+dA] in time interval dt is given by

        n(A, t) dA dt = a(t) A^(-2) dA dt,

    where A is the "initial" bipole area in square degrees, and t is the time
    in days; a(t) varies from 1.23 at cycle minimum to 10 at cycle maximum.

    The bipole area is the area with the 25-Gauss contour in the "initial"
    state, i.e., at the time of maximum development of the active region.
    The assumed peak flux density in the initial state is 100 G, and
    width = 0.4*bsiz.

    """
    amplitude = 10 * activity_rate
    cycle_length_days = cycle_length * YEAR2DAY
    nclen = (cycle_length + cycle_overlap) * YEAR2DAY

    # tau is time since last emergence in each lat/lon bin
    tau = np.ones((nlon, nlat, 2), dtype=np.long) * tau2

    # width of latitude and longitude bins
    # Let latitude evolution for butterfly diagram go some amount
    # `lat_width` above max_ave_lat and below min_ave_lat
    lat_width = 7  # degrees
    lat_max = max_ave_lat + lat_width
    lat_min = max(min_ave_lat - lat_width, 1) # Using 1 instead of 0 avoids dividing by zero later
    dlat = (lat_max - lat_min) / nlat
    dlon = 360 / nlon

    # The fractional area taken up by the active latitude bands is
    # 2 * 2pi * delta_lat / 4pi = delta_lat
    # If we treat that entire area as a spot with a given contrast,
    # we find a floor for the flux modulation. If we hit that floor,
    # we suppress the formation of spots.  
    frac_area = np.sin(lat_max/RAD2DEG) - np.sin(lat_min/RAD2DEG)
    threshold_flux = spot_contrast*frac_area

    spots = DataFrame(columns=['nday', 'lat', 'lon', 'bmax'])

    Nday, Icycle = np.mgrid[0:tsim, 0:2].reshape(2, 2 * tsim)
    n_current_cycle = Nday // cycle_length_days
    Nc = n_current_cycle - Icycle
    Nstart = np.fix(cycle_length_days * Nc)
    phase = (Nday - Nstart) / nclen
    # Quick fix to handle phases > 1
    Nday[phase >= 1] = -1
    
    # Emergence rate of uncorrelated active regions, from
    # Schrijver & Harvey (1994)
    ru0_tot = amplitude * np.sin(np.pi * phase) ** 2 * dcon / max_area

    latavg, latrms = active_latitudes(
        min_ave_lat, max_ave_lat, phase, butterfly=butterfly
    )
    lat_bins = np.arange(nlat)
    lat_bins_matrix = np.outer(lat_bins, np.ones(len(Nday)))
    # Probability of spot emergence is gaussian-distributed about latavg with scatter latrms
    p = np.exp(-((lat_min + (lat_bins_matrix + 0.5) * dlat - latavg) / latrms) ** 2)

    for i_count, nday in enumerate(Nday):
        # Quick fix to handle phases > 1
        if nday == -1:
            continue
            
        tau += 1

        # Check if we're close to the threshold flux. 
        earliest = max(nday - 0.5*decay_time, 0)
        # flux removed corresponds to calculation of dF in spots.py.
        # field strength is normalized by median strength, 55.8 gauss
        total_flux_removed = alpha_med * spots.query(f'nday > {earliest}').bmax.sum()/ 55.8

        if total_flux_removed > threshold_flux: 
            continue

        # Emergence rate of correlated active regions
        rc0 = np.zeros((nlon, nlat, 2))
        index = (tau1 < tau) & (tau < tau2)
        if index.any():
            rc0[index] = prob / (tau2 - tau1)

        psum = p[:, i_count].sum()
        if psum == 0:
            ru0 = p[:, i_count]
        else:
            ru0 = ru0_tot[i_count] * p[:, i_count] / (2 * nlon * psum)

        # k = 0: Northern hemisphere; k = 1: Southern hemisphere
        for k in [0, 1]:
            for j in lat_bins:
                r0 = ru0[j] + rc0[:, j, k]
                rtot = r0.sum()
                sumv = rtot * ftot

                x = np.random.uniform()
                if sumv > x:  # emerge spot
                    # Add rtot*fact elements until the sum is greater than x
                    cum_sum = rtot * fact.cumsum()
                    nb = (cum_sum >= x).argmax()
                    if nb == 0:
                        sumb = 0
                    else:
                        sumb = cum_sum[nb - 1]

                    cum_sum = sumb + fact[nb] * r0.cumsum()
                    i = (cum_sum >= x).argmax()
                    sumb = cum_sum[i]

                    lon = dlon * (np.random.uniform() + i)
                    lat = max(lat_min + dlat * (np.random.uniform() + j), 0)

                    if nday > tstart:
                        # Eq. 15 from van Ballegooijen:
                        # B_r = B_max * (flux_width/width_threshold)^2, Bmax is
                        # solar-calibrated initial peak flux density in gauss
                        # if Bmax = 250, then Bmax*(flux_width/width_thresh)^2
                        # = 250 * (0.4 * bipole_width/4.0)^2
                        # = 250/100 * bipole_area
                        # = 2.5 * bipole_area
                
                        peak_magnetic_flux = 2.5 * areas[nb]
                        
                        lat_rad = lat / RAD2DEG
                        lon_rad = lon / RAD2DEG

                        spots = spots.append(
                            Series([nday, (1 - 2*k) * lat_rad, lon_rad, peak_magnetic_flux], spots.columns),
                            ignore_index=True
                        )

                        if nb < 1:
                            tau[i, j, k] = 0

    return spots



# pylint: disable=no-member
"""
spots.py
Contains the Spots class, which holds parameters for spots on a given star.
"""
import os

import numpy as np
import pandas as pd
from astropy.io import fits
import matplotlib.pyplot as plt
from matplotlib import animation, gridspec, patches

from .constants import RAD2DEG, DAY2SEC, PROT_SUN, FLUX_SCALE


def diffrot(omega0, delta_omega, lat):
    """
    Default differential rotation function:
    Gives angular velocity as a function of latitude.
    """
    return omega0 - delta_omega * np.sin(lat) ** 2


class Spots(object):
    """
    Holds parameters for spots on a given star.

    PARAMETERS
    ----------
    spot_properties

    dur

    alpha_med

    incl

    omega

    delta_omega

    diffrot_func

    tau_evol

    threshold

    """

    def __init__(
        self,
        spot_properties,
        dur=None,
        alpha_med=FLUX_SCALE,
        incl=1,
        period=PROT_SUN,
        diffrot_shear=0.2,
        diffrot_func=diffrot,
        decay_timescale=5.0,
        threshold=0.1,
    ):

        ## global stellar parameters, same for all spots
        self.spot_properties = spot_properties
        self.inclination = incl

        # rotation
        self.omega = 2 * np.pi / (period * DAY2SEC)
        self.delta_omega = diffrot_shear * self.omega
        self.diffrot_func = diffrot_func

        # spot emergence and decay timescales
        self.emergence_timescale = max(
            2.0, period * decay_timescale / 5
        )
        self.decay_timescale = period * decay_timescale

        # convert active region properties
        time = spot_properties["nday"].values
        latitude = spot_properties["lat"].values
        longitude = spot_properties["lon"].values
        peak_magnetic_flux = spot_properties["bmax"]

        # keep only spots emerging within specified timespan and with peak B-field > threshold
        if dur is None:
            self.dur = time.max()
        else:
            self.dur = dur

        keep = (time < self.dur) & (peak_magnetic_flux > threshold)
        self.nspot = keep.sum()
        self.t0 = time[keep]
        self.latitude = latitude[keep]
        self.longitude = longitude[keep]
        self.area_max = (
            alpha_med * (peak_magnetic_flux / np.median(peak_magnetic_flux))[keep]
        )
        # alpha is spot contrast * spot area

    def calc_i(self, time, animate=False):
        """Modulate flux for single time step"""
        tt = time - self.t0
        t_emerge = self.emergence_timescale
        t_decay = self.decay_timescale
        timescale = np.where(tt < 0, t_emerge, t_decay)
        current_area = self.area_max * np.exp(-0.5 * (tt / timescale) ** 2)

        omega_lat = self.diffrot_func(self.omega, self.delta_omega, self.latitude)
        current_lon = omega_lat * time * DAY2SEC + self.longitude

        cos_beta = np.cos(self.inclination) * np.sin(self.latitude) + np.sin(
            self.inclination
        ) * np.cos(self.latitude) * np.cos(current_lon)

        dF = -current_area * np.maximum(cos_beta, 0)

        if animate:
            return self.latitude, current_lon, current_area, dF
        return dF.sum()

    def calc(self, time, max_memory=1000):
        """Modulate flux using chunks for all spots"""
        M = self.nspot
        N = len(time)
        memory_ceil = max_memory * (1024 ** 2) # bytes

        if 8 * M * N < memory_ceil:
            dFlux = self.matrix_calc(time)
        else:
            nbins = int(np.ceil(8 * M * N / memory_ceil))
            n = int(N / nbins)
            dFlux = np.zeros(N)
            for i in range(nbins):
                dFlux[i * n : (i + 1) * n] = self.matrix_calc(time[i * n : (i + 1) * n])
            if nbins * n < N:
                dFlux[nbins * n :] = self.matrix_calc(time[nbins * n :])
        return dFlux

    def matrix_calc(self, time):
        """Modulate flux for all spots"""
        M = self.nspot
        N = len(time)

        # spot area
        tt = np.outer(np.ones(M), time) - np.outer(self.t0, np.ones(N))
        area = np.outer(self.area_max, np.ones(N))
        emerge = np.outer(self.emergence_timescale, np.ones(N))
        decay = np.outer(self.decay_timescale, np.ones(N))
        timescale = np.where(tt < 0, emerge, decay)
        area *= np.exp(-0.5 * (tt / timescale) ** 2)

        # rotation rate
        omega = self.diffrot_func(self.omega, self.delta_omega, self.latitude)
        # here 'phase' is the longitude as a function of time
        phase = np.outer(omega, time * DAY2SEC) + np.outer(self.longitude, np.ones(N))

        # foreshortening
        # if inclination is angle between pole and line of sight,
        # cos(beta) = cos(i)*sin(lat) + sin(i)*cos(lat)*cos(lon(t)).
        cos_beta = np.cos(self.inclination) * np.outer(
            np.sin(self.latitude), np.ones(N)
        ) + np.sin(self.inclination) * np.outer(
            np.cos(self.latitude), np.ones(N)
        ) * np.cos(
            phase
        )

        # Total modulation on stellar flux
        dFlux = -area * np.maximum(cos_beta, 0)

        # return area, omega, beta, dFlux
        return dFlux.sum(axis=0)

    def plot_butterfly(self, fig=None, ax=None):
        """Plot the butterfly diagram"""
        latitude = self.spot_properties["lat"]

        if fig is None:
            fig = plt.figure()
        if ax is None:
            ax = fig.add_subplot()

        ax.scatter(
            self.spot_properties["nday"],
            latitude * RAD2DEG,
            s=10
            * self.spot_properties["bmax"]
            / np.median(self.spot_properties["bmax"]),
            alpha=0.5,
            c="#996699",
            lw=0.5,
        )
        ax.set_xlim(0, self.spot_properties["nday"].max())
        ax.set_ylim(-90, 90)
        ax.set_yticks((-90, -45, 0, 45, 90))
        ax.set_xlabel("Time (days)")
        ax.set_ylabel("Latitude (deg)")

    def ortho_animation(self, time, lightcurve, window_size=50, 
        fig_kw={"figsize": (5, 6)}, **kw):
        try:
            import cartopy.crs as ccrs
        except:
            raise ImportError(
                "butterpy requires the use of cartopy to run orthographic animations, "
                "but cartopy is not installed. Please install using "
                "`conda install -c conda-forge cartopy`, or if using pip, see "
                "https://scitools.org.uk/cartopy/docs/latest/installing.html for details."
            )

        fig = plt.figure(**fig_kw)
        #fig.subplots_adjust(
        #    top=0.95, bottom=0.08, left=0.16, right=0.95, hspace=0.1)
        gs = gridspec.GridSpec(2, 1, figure=fig, height_ratios=(1, 0.5))
        ax1 = fig.add_subplot(gs[0], 
            projection=ccrs.Orthographic(0, 90 - RAD2DEG * self.inclination))
        ax2 = fig.add_subplot(gs[1])

        ax1.set_global()
        ax1.gridlines(color='k', linestyle='dotted')
        ax1.scatter([], [], s=1, alpha=0.5, c="#996699", lw=0.5,
            transform=ccrs.PlateCarree())

        ax2.plot(lightcurve["time"], lightcurve["flux"])
        ax2.add_artist(
            patches.Ellipse(
                (0.5, 0.5),
                width=0.02,
                height=0.05,
                color="r",
                transform=ax2.transAxes,
                zorder=2,
            )
        )
        width = window_size / 2
        ax2.set_ylim(0.97, 1)
        ax2.set_xticks(np.arange(time[0] - width / 2, time[-1] + width / 2, width / 2))
        ax2.set_xlim(time[0] - width, time[0] + width)
        ax2.set_xlabel("Time (days)")
        ax2.set_ylabel("Relative Flux")
        ax2.vlines(0.5, color="r", ymin=0, ymax=1, transform=ax2.transAxes)
        title = fig.suptitle("")
        fig.align_labels()

        ax = (ax1, ax2)
        ani = animation.FuncAnimation(
            fig,
            _update_figure,
            frames=time,
            blit=False,
            repeat=False,
            fargs=(self, ax, title, 'ortho'),
            **kw,
        )
        return ani

    def animate_evolution(self, time, sim, lightcurve, window_size=50, **kw):
        nlat = 16
        nlon = 36
        dlon = 360 / nlon

        lat_width = 7
        lat_min = max(sim["Spot Min"] - lat_width, 0)
        lat_max = sim["Spot Max"] + lat_width

        fig = plt.figure(figsize=(5, 6), constrained_layout=False)
        fig.subplots_adjust(
            top=0.95, bottom=0.08, left=0.16, right=0.95, hspace=0.3, wspace=0.2
        )
        gs = gridspec.GridSpec(2, 1, figure=fig, height_ratios=(1, 0.5))
        ax1 = fig.add_subplot(gs[0])
        ax2 = fig.add_subplot(gs[1])

        ax1.set_xlim(-180, 180)
        ax1.set_xticks(np.linspace(-180, 180, 7))
        ymax = np.ceil(lat_max / 5) * 5
        ax1.set_ylim(-ymax, ymax)
        ax1.set_yticks(np.linspace(-ymax, ymax, 5))
        ax1.xaxis.set_ticklabels([180, 240, 300, 0, 60, 120, 180])

        ax1.vlines(
            np.linspace(dlon - 180, 180 - dlon, nlon - 1),
            ymin=lat_min,
            ymax=lat_max,
            lw=0.2,
        )
        ax1.vlines(
            np.linspace(dlon - 180, 180 - dlon, nlon - 1),
            ymin=-lat_max,
            ymax=-lat_min,
            lw=0.2,
        )
        ax1.hlines(np.linspace(lat_min, lat_max, nlat + 1), xmin=-180, xmax=180, lw=0.2)
        ax1.hlines(
            -np.linspace(lat_min, lat_max, nlat + 1), xmin=-180, xmax=180, lw=0.2
        )

        ax1.set_xlabel("Longitude (degrees)")
        ax1.set_ylabel("Latitude (degrees)")
        ax1.scatter([], [], s=1, alpha=0.5, c="#996699", lw=0.5)

        ax2.plot(lightcurve["time"], lightcurve["flux"])
        ax2.add_artist(
            patches.Ellipse(
                (0.5, 0.5),
                width=0.02,
                height=0.05,
                color="r",
                transform=ax2.transAxes,
                zorder=2,
            )
        )
        width = window_size / 2
        ax2.set_ylim(0.97, 1)
        ax2.set_xticks(np.arange(time[0] - width / 2, time[-1] + width / 2, width / 2))
        ax2.set_xlim(time[0] - width, time[0] + width)
        ax2.set_xlabel("Time (days)")
        ax2.set_ylabel("Relative Flux")
        ax2.vlines(0.5, color="r", ymin=0, ymax=1, transform=ax2.transAxes)
        title = fig.suptitle("")
        fig.align_labels()

        ax = (ax1, ax2)
        ani = animation.FuncAnimation(
            fig,
            _update_figure,
            frames=time,
            blit=False,
            repeat=False,
            fargs=(self, ax, title),
            **kw,
        )
        return ani


def _update_figure(time, spots, ax, title, projection=None):
    ax1, ax2 = ax

    latitudes, longitudes, areas, fluxes = spots.calc_i(time, animate=True)
    longitudes = (longitudes * RAD2DEG) % 360
    longitudes[longitudes >= 180] -= 360

    if projection == 'ortho':
        coll_idx = 0
        sizes = -10 * fluxes / FLUX_SCALE
    else:
        coll_idx = -1
        sizes = 10 * areas / FLUX_SCALE

    im1 = ax1.collections[coll_idx]
    im1.set_offsets(np.c_[longitudes, latitudes * RAD2DEG])
    im1.set_sizes(sizes)
    new_flux = 1 + fluxes.sum()

    y1, y2 = ax2.get_ylim()
    if new_flux < y1 + 0.01:
        y1 = new_flux - 0.01
        ax2.set_ylim(y1, y2)
    circle_y = (new_flux - y1) / (y2 - y1)
    circle, = ax2.artists
    circle.set_center((0.5, circle_y))

    t1, t2 = ax2.get_xlim()
    t0 = (t1 + t2) / 2
    delta_t = time - t0
    ax2.set_xlim(t1 + delta_t, t2 + delta_t)

    time_minutes = time * 24 * 60
    hours, minutes = divmod(time_minutes, 60)
    days, hours = divmod(hours, 24)
    title.set_text(f"Elapsed: {days:4.0f}d{hours:02.0f}h{minutes:02.0f}m")
    return (im1,)


def get_animation(path, time, projection=None, verbose=False, **kw):
    root, fname = os.path.split(path)
    sim_number = int("".join([char for char in fname if char.isdigit()]))
    path_to_sims = os.path.join(root, 'simulation_properties.csv')

    my_sim = pd.read_csv(path_to_sims).iloc[sim_number]

    if verbose:
        for label, item in my_sim.iteritems():
            print(f'{label.ljust(20, ".")}{item}')

    with fits.open(path) as f:
        lightcurve = f[1].data
        spot_properties = f[2].data

    my_spots = Spots(
        pd.DataFrame(spot_properties),
        incl=my_sim["Inclination"],
        period=my_sim["Period"],
        diffrot_shear=my_sim["Shear"],
        alpha_med=np.sqrt(my_sim["Activity Rate"]) * FLUX_SCALE,
        decay_timescale=my_sim["Decay Time"],
    )

    if projection == 'ortho':
        ani = my_spots.ortho_animation(time, lightcurve, **kw)
    else:
        ani = my_spots.animate_evolution(time, my_sim, lightcurve, **kw)
    return ani


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    readme = fh.read()

exec(open("butterpy/version.py").read())

setup(
    name="butterpy",
    version=__version__,
    author="Zachary R. Claytor <zclaytor@hawaii.edu> and Miles Lucas <mdlucas@hawaii.edu>",
    description="Tools for simulating stellar rotational light curves using realistic spot evolution",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/zclaytor/butterpy",
    license="MIT",
    python_requires="==3.*,>=3.6.0",
    packages=find_packages(),
    install_requires=["astropy", "numpy", "scipy", "matplotlib", "tqdm", "pandas"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Astronomy",
    ],
)


# constants.py
from numpy import pi

DAY2MIN = 24 * 60  # days to minutes
DAY2SEC = DAY2MIN * 60  # days to seconds
YEAR2DAY = 365  # years to days
RAD2DEG = 180 / pi  # radians to degrees

PROT_SUN = 24.5  # siderial rotation period at the equator
OMEGA_SUN = 2 * pi / (PROT_SUN * DAY2SEC)

# Realized 6/26/2020 that this is 1 square degree in radians
FLUX_SCALE = 3e-4  # Scaled to Sun 
